@@ -48,7 +48,6 @@ from .braille_generator import BrailleGenerator
 from .formatting import Formatting
 from .structural_navigation import StructuralNavigation
 from .script_utilities import Utilities
-from . import script_settings
 
 _settingsManager = settings_manager.getManager()
 
@@ -63,10 +62,7 @@ class Script(default.Script):
 
         default.Script.__init__(self, app)
 
-        self.savedEnabledBrailledTextAttributes = None
-        self.savedEnabledSpokenTextAttributes = None
         self.speakSpreadsheetCoordinatesCheckButton = None
-        self.savedreadTableCellRow = None
         self.skipBlankCellsCheckButton = None
         self.speakCellCoordinatesCheckButton = None
         self.speakCellHeadersCheckButton = None
@@ -90,49 +86,6 @@ class Script(default.Script):
         self.lastStartOff = -1
         self.lastEndOff = -1
 
-    def activate(self):
-        """Called when this script is activated."""
-        self.savedreadTableCellRow = \
-            _settingsManager.getSetting('readTableCellRow')
-        _settingsManager.setSetting('readTableCellRow', False)
-
-        self.savedEnabledBrailledTextAttributes = \
-            _settingsManager.getSetting('enabledBrailledTextAttributes')
-
-        self.savedEnabledSpokenTextAttributes = \
-            _settingsManager.getSetting('enabledSpokenTextAttributes')
-
-        # Account for the differences in how OOo expresses indent,
-        # strikethrough, and margins.
-        #
-        attributes = _settingsManager.getSetting('allTextAttributes')
-        attributes.replace("margin:;", "margin:0mm;")
-        _settingsManager.setSetting('allTextAttributes', attributes)
-
-        attributes = \
-            _settingsManager.getSetting('enabledBrailledTextAttributes')
-        attributes.replace("strikethrough:false;", "strikethrough:none;")
-        attributes.replace("indent:0;", "indent:0mm;")
-        _settingsManager.setSetting('enabledBrailledTextAttributes', attributes)
-
-        attributes = _settingsManager.getSetting('enabledSpokenTextAttributes')
-        attributes.replace("strikethrough:false;", "strikethrough:none;")
-        attributes.replace("indent:0;", "indent:0mm;")
-        _settingsManager.setSetting('enabledSpokenTextAttributes', attributes)
-
-        default.Script.activate(self)
-
-    def deactivate(self):
-        """Called when this script is deactivated."""
-        _settingsManager.setSetting('readTableCellRow',
-                                    self.savedreadTableCellRow)
-        _settingsManager.setSetting('enabledBrailledTextAttributes',
-                                    self.savedEnabledBrailledTextAttributes)
-        _settingsManager.setSetting('enabledSpokenTextAttributes',
-                                    self.savedEnabledSpokenTextAttributes)
-
-        default.Script.activate(self)
-
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
@@ -223,39 +176,39 @@ class Script(default.Script):
         keyBindings.add(
             keybindings.KeyBinding(
                 "a",
-                settings.defaultModifierMask,
-                settings.ORCA_MODIFIER_MASK,
+                keybindings.defaultModifierMask,
+                keybindings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["presentInputLineHandler"]))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "r",
-                settings.defaultModifierMask,
-                settings.ORCA_MODIFIER_MASK,
+                keybindings.defaultModifierMask,
+                keybindings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["setDynamicColumnHeadersHandler"],
                 1))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "r",
-                settings.defaultModifierMask,
-                settings.ORCA_MODIFIER_MASK,
+                keybindings.defaultModifierMask,
+                keybindings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["clearDynamicColumnHeadersHandler"],
                 2))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "c",
-                settings.defaultModifierMask,
-                settings.ORCA_MODIFIER_MASK,
+                keybindings.defaultModifierMask,
+                keybindings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["setDynamicRowHeadersHandler"],
                 1))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "c",
-                settings.defaultModifierMask,
-                settings.ORCA_MODIFIER_MASK,
+                keybindings.defaultModifierMask,
+                keybindings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["clearDynamicRowHeadersHandler"],
                 2))
 
@@ -273,7 +226,7 @@ class Script(default.Script):
         grid.set_border_width(12)
 
         label = guilabels.SPREADSHEET_SPEAK_CELL_COORDINATES
-        value = script_settings.speakSpreadsheetCoordinates
+        value = _settingsManager.getSetting('speakSpreadsheetCoordinates')
         self.speakSpreadsheetCoordinatesCheckButton = \
             Gtk.CheckButton.new_with_mnemonic(label)
         self.speakSpreadsheetCoordinatesCheckButton.set_active(value)
@@ -324,60 +277,16 @@ class Script(default.Script):
 
         return grid
 
-    def setAppPreferences(self, prefs):
-        """Write out the application specific preferences lines and set the
-        new values.
+    def getPreferencesFromGUI(self):
+        """Returns a dictionary with the app-specific preferences."""
 
-        Arguments:
-        - prefs: file handle for application preferences.
-        """
-
-        prefs.writelines("\n")
-        prefix = "orca.scripts.apps.soffice.script_settings"
-        prefs.writelines("import %s\n\n" % prefix)
-
-        script_settings.speakSpreadsheetCoordinates = \
-            self.speakSpreadsheetCoordinatesCheckButton.get_active()
-        prefs.writelines("%s.speakSpreadsheetCoordinates = %s\n" % \
-                        (prefix, script_settings.speakSpreadsheetCoordinates))
-
-        value = self.speakCellCoordinatesCheckButton.get_active()
-        _settingsManager.setSetting('speakCellCoordinates', value)
-        prefs.writelines("orca.settings.speakCellCoordinates = %s\n" % value)
-
-        value = self.speakCellSpanCheckButton.get_active()
-        _settingsManager.setSetting('speakCellSpan', value)
-        prefs.writelines("orca.settings.speakCellSpan = %s\n" % value)
-
-        value = self.speakCellHeadersCheckButton.get_active()
-        _settingsManager.setSetting('speakCellHeaders', value)
-        prefs.writelines("orca.settings.speakCellHeaders = %s\n" % value)
-
-        value = self.skipBlankCellsCheckButton.get_active()
-        _settingsManager.setSetting('skipBlankCells', value)
-        prefs.writelines("orca.settings.skipBlankCells = %s\n" % value)
-
-    def getAppState(self):
-        """Returns an object that can be passed to setAppState.  This
-        object will be use by setAppState to restore any state information
-        that was being maintained by the script."""
-        return [default.Script.getAppState(self),
-                self.dynamicColumnHeaders,
-                self.dynamicRowHeaders]
-
-    def setAppState(self, appState):
-        """Sets the application state using the given appState object.
-
-        Arguments:
-        - appState: an object obtained from getAppState
-        """
-        try:
-            [defaultAppState,
-             self.dynamicColumnHeaders,
-             self.dynamicRowHeaders] = appState
-            default.Script.setAppState(self, defaultAppState)
-        except:
-            debug.printException(debug.LEVEL_WARNING)
+        return {
+            'speakCellSpan': self.speakCellSpanCheckButton.get_active(),
+            'speakCellHeaders': self.speakCellHeadersCheckButton.get_active(),
+            'skipBlankCells': self.skipBlankCellsCheckButton.get_active(),
+            'speakCellCoordinates': self.speakCellCoordinatesCheckButton.get_active(),
+            'speakSpreadsheetCoordinates': self.speakSpreadsheetCoordinatesCheckButton.get_active(),
+        }
 
     def isStructuralNavigationCommand(self, inputEvent=None):
         """Checks to see if the inputEvent was a structural navigation
@@ -729,8 +638,8 @@ class Script(default.Script):
                      pyatspi.ROLE_ROOT_PANE,
                      pyatspi.ROLE_FRAME,
                      pyatspi.ROLE_APPLICATION]
-        if self.utilities.hasMatchingHierarchy(event.source, rolesList):
-            for child in event.source.parent:
+        if self.utilities.hasMatchingHierarchy(newLocusOfFocus, rolesList):
+            for child in newLocusOfFocus.parent:
                 if child.getRole() == pyatspi.ROLE_PAGE_TAB_LIST:
                     for tab in child:
                         eventState = tab.getState()
@@ -754,8 +663,8 @@ class Script(default.Script):
             # But this will get us to speak the entire paragraph when navigation by
             # paragraph has occurred.
             event_string, mods = self.utilities.lastKeyAndModifiers()
-            isControlKey = mods & settings.CTRL_MODIFIER_MASK
-            isShiftKey = mods & settings.SHIFT_MODIFIER_MASK
+            isControlKey = mods & keybindings.CTRL_MODIFIER_MASK
+            isShiftKey = mods & keybindings.SHIFT_MODIFIER_MASK
             if event_string in ["Up", "Down"] and isControlKey and not isShiftKey:
                 if self.utilities.displayedText(newLocusOfFocus):
                     speech.speak(self.utilities.displayedText(newLocusOfFocus))
@@ -942,14 +851,23 @@ class Script(default.Script):
         if event.source.getRoleName() == 'text frame':
             return
 
-        ignoreRoles = [pyatspi.ROLE_FILLER, pyatspi.ROLE_PANEL]
-        if event.source.getRole() in ignoreRoles:
-            return
-
         parent = event.source.parent
         if parent and parent.getRoleName() == 'text frame':
             return
 
+        if parent and parent.getRole() == pyatspi.ROLE_TOOL_BAR:
+            default.Script.onFocusedChanged(self, event)
+            return
+
+        role = event.source.getRole()
+        ignoreRoles = [pyatspi.ROLE_FILLER, pyatspi.ROLE_PANEL]
+        if role in ignoreRoles:
+            return
+
+        # We will present this when the selection changes.
+        if role == pyatspi.ROLE_MENU:
+            return
+
         obj, offset = self.pointOfReference.get("lastCursorPosition", (None, -1))
         textSelections = self.pointOfReference.get('textSelections', {})
         start, end = textSelections.get(hash(obj), (0, 0))
@@ -959,6 +877,12 @@ class Script(default.Script):
         if self.utilities._flowsFromOrToSelection(event.source):
             return
 
+        if role == pyatspi.ROLE_PARAGRAPH:
+            keyString, mods = self.utilities.lastKeyAndModifiers()
+            if keyString in ["Left", "Right"]:
+                orca.setLocusOfFocus(event, event.source, False)
+                return
+
         # We should present this in response to active-descendant-changed events
         if event.source.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS):
             return
@@ -1027,11 +951,13 @@ class Script(default.Script):
             x = orca_state.lastInputEvent.x
             y = orca_state.lastInputEvent.y
             weToggledIt = obj.queryComponent().contains(x, y, 0)
+        elif obj.getState().contains(pyatspi.STATE_FOCUSED):
+            weToggledIt = True
         else:
             keyString, mods = self.utilities.lastKeyAndModifiers()
             navKeys = ["Up", "Down", "Left", "Right", "Page_Up", "Page_Down",
-                       "Home", "End"]
-            wasCommand = mods & settings.COMMAND_MODIFIER_MASK
+                       "Home", "End", "N"]
+            wasCommand = mods & keybindings.COMMAND_MODIFIER_MASK
             weToggledIt = wasCommand and keyString not in navKeys
         if weToggledIt:
             speech.speak(self.speechGenerator.generateSpeech(obj))
